package com.ibm.mqe.model;

/****************************************************************************/
/*                                                                          */
/* Program name: MQ Series Everyplace - Administration Tool                 */
/*                                                                          */
/*  Statement:  Licensed Materials - Property of IBM                        */
/*                                                                          */
/*              MQSeries SupportPac MSE1                                    */
/*              (C) Copyright IBM Corp. 2000                                */
/*                                                                          */
/****************************************************************************/
/*                                                                          */
/* Change history:                                                          */
/*                                                                          */
/* V1.0   15-08-2000  PTvL  Initial release                                 */
/*                                                                          */
/*==========================================================================*/
import java.util.*;
import com.ibm.mqe.*;
import com.ibm.mqe.mqbridge.*;

import com.ibm.mqe.administration.*;

/**
 * This type represents an administered MQeQueueManager.
 * Each instance of this type will own and manage an instance of
 * <code>MQeAdminConnection</code> which represents the MQe resources
 * required to pass admin messages to the MQeQueueManager represented.
 * Creation date: (19/05/2000 14:30:21)
 * @author: Phill van Leersum
 */
public class IndirectQueueManagerStub extends MQeQueueManagerStub implements Runnable{
	private Thread thread;
	private boolean createdOutConnection = false;
	
	private int connectionMode = CONNECTION_MODE_DISCONNECTED;
	
	/*
	 * The address of the route (ie adaptor information
	 */
	private String routeAddress;
	private Hashtable pendingMessages;
/**
 * MQeQueueManager constructor.  This is made protected to guide users
 * to the creation method supplied in MQeAdministeringQM.  Honouring
 * the factory pattern will enable developers to extend the capabilities
 * of this code by subclassing (rather than by editing).
 * @param metaObject com.ibm.mq.mmodel.MQeAdministeringQM
 */
public IndirectQueueManagerStub(MQeStubMessageListener listener, MQeQueueManager adminQM) {
	super(listener, adminQM);
	this.connectionMode = CONNECTION_MODE_DISCONNECTED;
	pendingMessages = new Hashtable();
}
/**
 * create a route FROM the admin queue manager TO the queue
 * manager represented by the receiver.  reply to the the 
 * admin queue manager
 */
private final boolean _addAdminOutRoute(String route) {
	if (!createdOutConnection) {
		MQeAdminMsg replyMsg = _addRoute(getName(), route, "Administration Out Route");
		if (null == replyMsg) {
			_error("No reply while creating admin out route");
		} else {
			if (_isSuccess(replyMsg)) {
				createdOutConnection = true;
			} else if (MQe.Except_QMgr_AlreadyExists == _getReasonCode(replyMsg)) {
				createdOutConnection = true;
			} else {
				_error("Error creating Out Route: " + _getReason(replyMsg));
			}
		}
	}
	return createdOutConnection;
}
/**
 * Add a route on a specified machine to the specified queueManager,
 * via the given address.
 * toQM: name of qm to which the route points
 * routeAddress:  address of machine/port upon which the QM is listening
 * toQM: name of qm from which the route points (ie QM where route is created)
 * replyQM: name of queueManager to reply to (null = no reply)
 * description: initial description
 */
private final MQeAdminMsg _addRoute(String toQM, String routeAddress, String description) {
	MQeAdminMsg replyMsg = null;
	try {
		MQeConnectionAdminMsg msg = new MQeConnectionAdminMsg();
		// Set name of queue manager to add routes to
		msg.setName(toQM);
		// Set the admin action to create a new queue
		msg.create(routeAddress, null, null, "DefaultChannel", description);
		// The local queue manager will process this message
		//msg.setTargetQMgr(getAdminQueueManagerName());
		replyMsg = _synchronousPutLocalAdminMessage(msg);
	} catch (Exception e) {
	}
	return replyMsg;
}
/**
 * put an admin message.
 *
 * If no listener has been set then the method waits for a response to 
 * the admin message, and returns it.
 * If a listener has been set, then the message will be returned
 * asynchronously via the listeners acceptMessage() method.
 *
 * This implementation of the method adds the queueManager specific
 * data to the request and passes it to the administrating queueManager.
 */
private final void _asynchronousPutAdminMessage(MQeAdminMsg msg, MQeStubMessageListener listener) {
	try {
		msg.setTargetQMgr(getName());
		String correlationId = _addReplyRequest(msg, getName());
		// for asynchronous requests we must remember who the message is for
		if (null != listener) {
			pendingMessages.put(correlationId, listener);
		}
		getAdminQueueManager().putMessage(getName(), MQe.Admin_Queue_Name, msg, null, 0);
	} catch (Exception e) {
		_error("Error putting async message: ", e);
	}
}
/**
 * if an out-bound connection has been created, then delete it
 */
private final void _removeAdminOutRoute() {
	if (createdOutConnection) {
		MQeAdminMsg replyMsg = _removeRoute(getName());
		if (_isSuccess(replyMsg)) {
			createdOutConnection = false;
		} else {
			if (MQe.Except_NotFound == _getReasonCode(replyMsg)) {
				_comment("Back Route already deleted ");
				createdOutConnection = false;
			} else {
				_error("Error removing Out Route: " + _getReason(replyMsg));
			}
		}
	}
}
/**
 * Remove a route on a specified machine to the specified queueManager,
 * via the given address.
 * toQM: name of qm to which the route points
 * fromQM: name of qm from which the route points (ie QM where route is created)
 * replyQM: name of queueManager to reply to (null = no reply)
 */
public final MQeAdminMsg _removeRoute(String toQM) {
	MQeAdminMsg replyMsg = null;
	try {
		MQeConnectionAdminMsg msg = new MQeConnectionAdminMsg();
		// Set name of queue manager to which the route points
		msg.setName(toQM);
		// Set the admin action to create a new queue
		msg.delete(new MQeFields());
		// set the queue manager which will process this message
		msg.setTargetQMgr(getAdminQueueManagerName());
		replyMsg = _synchronousPutAdminMessage(msg);
	} catch (Exception e) {
		_error("Error while removing route", e);
	}
	return replyMsg;
}
/**
 * put an admin message.
 *
 * If no listener has been set then the method waits for a response to 
 * the admin message, and returns it.
 * If a listener has been set, then the message will be returned
 * asynchronously via the listeners acceptMessage() method.
 *
 * This implementation of the method adds the queueManager specific
 * data to the request and passes it to the administrating queueManager.
 */
private final MQeAdminMsg _synchronousPutAdminMessage(MQeAdminMsg msg) {
	MQeAdminMsg replyMsg = null;
	try {
		msg.setTargetQMgr(getName());
		String correlationId = _addReplyRequest(msg, getName());
		getAdminQueueManager().putMessage(getName(), MQe.Admin_Queue_Name, msg, null, 0);
		replyMsg = _waitForMessage(getAdminQueueManager(), getName(), correlationId);
	} catch (Exception e) {
		_error("Error putting synchronous message: ", e);
	}
	return replyMsg;
}
/**
 * put an admin message.
 *
 * If no listener has been set then the method waits for a response to 
 * the admin message, and returns it.
 * If a listener has been set, then the message will be returned
 * asynchronously via the listeners acceptMessage() method.
 *
 * This implementation of the method adds the queueManager specific
 * data to the request and passes it to the administrating queueManager.
 */
private final MQeAdminMsg _synchronousPutLocalAdminMessage(MQeAdminMsg msg) {
	MQeAdminMsg replyMsg = null;
	try {
		String str = getAdminQueueManagerName();
		System.out.println("AdminQM Name = " + str);
		msg.setTargetQMgr(getAdminQueueManagerName());
		String correlationId = _addReplyRequest(msg, getAdminQueueManagerName());
		getAdminQueueManager().putMessage(getAdminQueueManagerName(), MQe.Admin_Queue_Name, msg, null, 0);
		replyMsg = _waitForMessage(getAdminQueueManager(), getAdminQueueManagerName(), correlationId);
	} catch (Exception e) {
		_error("Error putting synchronous message: ", e);
	}
	return replyMsg;
}
public boolean[] addTemporaryRoute(String destination, String route) {
	boolean[] reply = new boolean[] {false, false};
	_comment("Creating temporary route");
	
	MQeAdminMsg replyMsg = _addRoute(destination, route, "Temporary Out Route");
	if (null != replyMsg) {
		if (_isSuccess(replyMsg)) {
			reply[0] = true;
			reply[1] = true;
			_comment("Temporary route created");
		} else
			if (MQe.Except_QMgr_AlreadyExists == _getReasonCode(replyMsg)) {
				reply[0] = true;
				_comment("Temporary route already exists");
			} else {
				_warning("Error creating temporary Route: " + _getReason(replyMsg));
			}
	}
	return reply;
}
/**
 * browse messages assciated with the receiver
 */
public MQeEnumeration browseQueue(String queueQueueManagerName, String queueName) {
	MQeEnumeration enum = null;
	try {
		enum = getAdminQueueManager().browseMessages(
			queueQueueManagerName, queueName, null, null, false);
		
	} catch (Exception e) {
		_warning("Error browsing queue", e);
	}
	return enum;
}
public final boolean connect(String name, String route, int mode) {
	setName(name);
	if (_addAdminOutRoute(route)) {
		if (isConnected()) {
			disconnect(false);
			connectionMode = 0;
		}
		switch (mode) {
			case CONNECTION_MODE_ASYNCHRONOUS :
				if (null == thread) {
					thread = new Thread(this);
					thread.start();
				}
				break;
			case CONNECTION_MODE_SYNCHRONOUS :
				break;
			default :
				break;
		}
		connectionMode = mode;
	}
	return connectionMode == mode;
}
public boolean[] createTemporaryRemoteQueue(String queueQueueManagerName, String queueName) {
	boolean[] reply = new boolean[] {false, false};
	_comment("Creating temporary remote queue");
	try {
		MQeQueueAdminMsg msg = new MQeRemoteQueueAdminMsg();
		msg.setName(queueQueueManagerName, queueName); // name of resource
		msg.create(new MQeFields());
		MQeAdminMsg replyMsg = _synchronousPutAdminMessage(msg);
		if (null != replyMsg) {
			if (_isSuccess(replyMsg)) {
				reply[0] = true;
				reply[1] = true;
				_comment("Temporary remote queue created");
			} else
				if (MQe.Except_QMgr_AlreadyExists == _getReasonCode(replyMsg)) {
					reply[0] = true;
					_comment("Temporary remote queue already exists");
				} else {
					_warning("Error creating temporary remote queue: " + _getReason(replyMsg));
				}
		}
	} catch (Exception e) {
		// error creating queueAdminMessage
		//e.printStackTrace(_log());
	}
	return reply;
}
public boolean deleteMessage(String queueQueueManagerName, String queueName, MQeFields msg) {
	try {
		getAdminQueueManager().deleteMessage(
			queueQueueManagerName, queueName, msg);
		return true;
	} catch (Exception e) {
		_warning("error deleting message from "+getName()+"/"+ getName(), e);
	}
	return false;
}
public void deleteTemporaryRemoteQueue(String queueQueueManagerName, String queueName) {
	_comment("Deleting temporary remote queue");
	try {
		MQeQueueAdminMsg msg = new MQeRemoteQueueAdminMsg();
		msg.setName(queueQueueManagerName, queueName); // name of resource
		msg.delete(new MQeFields());
		MQeAdminMsg replyMsg = _synchronousPutAdminMessage(msg);
		if (null != replyMsg) {
			if (_isSuccess(replyMsg)) {
				_comment("Temporary remote queue deleted");
			} else {
				if (MQe.Except_QMgr_QDoesNotExist == _getReasonCode(replyMsg)) {
					_comment("Temporary remote queue already deleted");
				} else {
					_warning("Error deleting temporary remote queue: " + _getReason(replyMsg));
				}
			}
		}
	} catch (Exception e) {
		// error creating queueAdminMessage
		//e.printStackTrace(_log());
	}
}
public final boolean disconnect(boolean removeResources) {
	switch (connectionMode) {
		case CONNECTION_MODE_ASYNCHRONOUS :
			//if (null != thread) {
			//	thread.stop();
			//	thread = null;
			//}
			break;
		case CONNECTION_MODE_SYNCHRONOUS :
			break;
		default :
			break;
	}
	connectionMode = CONNECTION_MODE_DISCONNECTED;
	return !isConnected();
}
/**
 * Reply the instance of <code>MQeStubMessageListener</code>, if any,
 * registered as waiting for the given <code>MQeAdminMsg</code> reply
 * as defined by the correlation ID field.
 * If there is no registered listener, return null.
 */
public final MQeStubMessageListener getListenerForMessage(MQeAdminMsg msg) {
	MQeStubMessageListener li = null;
	try {
		String match = new String(msg.getArrayOfByte( MQe.Msg_CorrelID ));
		li = (MQeStubMessageListener)pendingMessages.remove(match);
	} catch (Exception ex) {
		System.out.println("Error during message event");
		ex.printStackTrace();
	}
	return li;
}
public final boolean isConnected() {
	return CONNECTION_MODE_DISCONNECTED != connectionMode;
}
/**
 * put an admin message.
 *
 * If no listener has been set then the method waits for a response to 
 * the admin message, and returns it.
 * If a listener has been set, then the message will be returned
 * asynchronously via the listeners acceptMessage() method.
 *
 * This implementation of the method adds the queueManager specific
 * data to the request and passes it to the administrating queueManager.
 */
public final MQeAdminMsg putAdminMessage(MQeAdminMsg msg, MQeStubMessageListener listener) {
	MQeAdminMsg replyMsg = null;
	if (isConnected()) {
		if (CONNECTION_MODE_ASYNCHRONOUS == connectionMode) {
			_asynchronousPutAdminMessage(msg, listener);
		} else {
			replyMsg = _synchronousPutAdminMessage(msg);
		}
	}
	return replyMsg;
}
public final void removeTemporaryRoute(String destination) {
	_comment("Deleting Back Route");
	MQeAdminMsg replyMsg = _removeRoute(destination);
	if (_isSuccess(replyMsg)) {
		_comment("Back Route deleted ");
	} else {
		if (MQe.Except_NotFound == _getReasonCode(replyMsg)) {
			_comment("Back Route already deleted ");
		} else {
			_comment("Error removing Temporary Route: " + _getReason(replyMsg));
		}
	}
}
public void run() {
	_comment("thread started");
	while (true) {
		MQeAdminMsg msg = null;
		if (CONNECTION_MODE_ASYNCHRONOUS == connectionMode) {
			do {
				//_comment("polling for message");
				msg = tryToGetMsg(null);
				if (null != msg) {
					//System.out.println("retreived msg");
					MQeStubMessageListener li = getListenerForMessage(msg);
					if (null != li) {
						//System.out.println("found listener");
						li.acceptMessage(msg);
					}
				}
				
			} while (null != msg);
		}
		try {
			Thread.sleep(100);
		} catch (InterruptedException e) {
		}
	}
}
/**
 * browse messages assciated with the receiver
 */
public void testMessage(String queueQueueManagerName, String queueName, String txt) {
	try {
		MQeMsgObject msg = new MQeMsgObject();
		msg.putAscii("Contents", txt);
		getAdminQueueManager().putMessage(
			queueQueueManagerName, queueName, msg, null, 0);
		
	} catch (Exception e) {
		_warning("Error placing test message", e);
	}
}
private MQeAdminMsg tryToGetMsg(MQeFields filter) {
	MQeAdminMsg msg = null;
	try {
		msg = (MQeAdminMsg)getAdminQueueManager().getMessage(
								getName(), 
								getReplyToQueueName(), filter, null, 0);
	} catch (Exception e) {
		e.getMessage();
	}
	return msg;
}
}
